import json
import requests
from pact_test.either import *
from pact_test.utils.logger import debug


def get_pact(location):
    if location.startswith(('http', 'https')):
        return __get_pact_from_url(location)
    return __get_pact_from_file(location)


def __get_pact_from_file(filename):
    debug('Get pact from file "' + str(filename) + '"')
    try:
        with open(filename) as file_content:
            return Right(json.loads(file_content.read()))
    except Exception as e:
        return Left(str(e))


def __get_pact_from_url(url):
    debug('Get pact from URL "' + str(url) + '"')
    try:
        return Right(requests.get(url).json())
    except Exception as e:
        return Left(str(e))



from pact_test.either import *
from pact_test.constants import *


class ServiceConsumerTest(object):
    pact_uri = None
    honours_pact_with = None

    @property
    def states(self):
        for attr in dir(self):
            obj = getattr(self, attr)
            if callable(obj) and hasattr(obj, 'state'):
                yield obj

    def is_valid(self):
        if self.pact_uri is None:
            msg = MISSING_PACT_URI + __file__
            return Left(msg)
        if self.honours_pact_with is None:
            msg = MISSING_HONOURS_PACT_WITH + __file__
            return Left(msg)
        return Right(True)


def pact_uri(pact_uri_value):
    def wrapper(calling_class):
        setattr(calling_class, 'set_pact_uri',
                eval('set_pact_uri(calling_class, "' + pact_uri_value + '")'))
        return calling_class
    return wrapper


def set_pact_uri(self, pact_uri_value):
    self.pact_uri = pact_uri_value


def honours_pact_with(honours_pact_with_value):
    def wrapper(calling_class):
        setattr(calling_class, 'set_honours_pact_with',
                eval('set_honours_pact_with(calling_class, "' + honours_pact_with_value + '")'))
        return calling_class
    return wrapper


def set_honours_pact_with(self, honours_pact_with_value):
    self.honours_pact_with = honours_pact_with_value


def state(state_value):
    def decorate(function):
        function.state = state_value
        return function
    return decorate



import os
import json
from pact_test.utils.logger import *
from pact_test.config.config_builder import Config
from pact_test.utils.logger import log_consumers_test_results
from pact_test.utils.logger import log_providers_test_results
from pact_test.runners.service_consumers.test_suite import ServiceConsumerTestSuiteRunner
from pact_test.runners.service_providers.test_suite import ServiceProviderTestSuiteRunner


def verify(verify_consumers=False, verify_providers=False):
    config = Config()

    if verify_consumers:
        run_consumer_tests(config)
    if verify_providers:
        run_provider_tests(config)


def run_consumer_tests(config):
    test_results = ServiceConsumerTestSuiteRunner(config).verify()
    log_consumers_test_results(test_results)


def run_provider_tests(config):
    test_results = ServiceProviderTestSuiteRunner(config).verify()
    log_providers_test_results(test_results)
    for pact in test_results.value:
        filename = pact['consumer']['name'] + '_' + pact['provider']['name'] + '.json'
        filename = filename.replace(' ', '_').lower()
        info('Writing pact to: ' + filename)
        with open(os.path.join(os.getcwd(), 'pacts', filename), 'w+') as file:
            file.write(json.dumps(pact))



import json
from pact_test.utils.logger import debug
from threading import Thread
from pact_test.models.response import PactResponse
try:
    import socketserver as SocketServer
    import http.server as SimpleHTTPServer
except ImportError:
    import SocketServer
    import SimpleHTTPServer


ARCHIVE = []


def build_proxy(mock_response=PactResponse()):
    class Proxy(SimpleHTTPServer.SimpleHTTPRequestHandler):
        def __init__(self, *args, **kwargs):
            SimpleHTTPServer.SimpleHTTPRequestHandler.__init__(self, *args, **kwargs)
            self.mock_response = mock_response

        def do_GET(self):
            data = self.read_request_data(self)
            self.handle_request('GET', data)

        def do_POST(self):
            data = self.read_request_data(self)
            self.handle_request('POST', data)

        def do_PUT(self):
            data = self.read_request_data(self)
            self.handle_request('PUT', data)

        def do_DELETE(self):
            data = self.read_request_data(self)
            self.handle_request('DELETE', data)

        @staticmethod
        def read_request_data(other_self):
            header_value = other_self.headers.get('Content-Length')
            data_length = int(header_value) if header_value is not None else None
            return other_self.rfile.read(data_length) if data_length is not None else None

        def format_request(self, http_method, data):
            path_and_query = self.path.split('?')
            path = path_and_query[0]
            query = '?' + path_and_query[1] if len(path_and_query) == 2 else ''

            return {
                'method': http_method,
                'path': path,
                'query': query,
                'body': json.loads(data.decode('utf-8')) if data is not None else data,
                'headers': list(dict([(key, value)]) for key, value in self.headers.items())
            }

        def handle_request(self, http_method, data):
            info = self.format_request(http_method, data)
            ARCHIVE.append(info)
            self.respond()

        def respond(self):
            self.send_response(int(mock_response.status))
            for header in mock_response.headers:
                for key, value in header.items():
                    self.send_header(key, value)
            self.end_headers()
            self.wfile.write(str(mock_response.body).encode())

    return Proxy


class ThreadedTCPServer(SocketServer.ThreadingMixIn, SocketServer.TCPServer):
    pass


class MockServer(object):

    def __init__(self, mock_response=PactResponse(), base_url='0.0.0.0', port=1234):
        self.port = port
        self.base_url = base_url
        self.server = ThreadedTCPServer((self.base_url, self.port), build_proxy(mock_response))
        self.server_thread = Thread(target=self.server.serve_forever)
        self.server_thread.daemon = True
        global ARCHIVE
        ARCHIVE = []

    def start(self):
        self.server_thread.start()
        debug('PROXY SERVER LISTENING ON http://' + self.base_url + ':' + str(self.port))

    def shutdown(self):
        debug('SHUTTING DOWN SERVER...')
        self.server.shutdown()
        self.server.server_close()
        debug('SHUTTING DOWN MOCK SERVER... DONE')

    @staticmethod
    def report():
        return ARCHIVE
